@@ -4,6 +4,7 @@ namespace Icinga\Module\Director\Controllers;
 
 use Exception;
 use Icinga\Exception\NotFoundError;
+use Icinga\Module\Director\Db\AppliedServiceSetLoader;
 use Icinga\Module\Director\Exception\NestingError;
 use Icinga\Module\Director\IcingaConfig\AgentWizard;
 use Icinga\Module\Director\Objects\IcingaHost;
@@ -133,7 +134,20 @@ class HostController extends ObjectController
 
         $this->addHostServiceSetTables($host, $tables);
         foreach ($parents as $parent) {
-            $this->addHostServiceSetTables($host, $tables);
+            $this->addHostServiceSetTables($parent, $tables, $host);
+        }
+
+        $appliedSets = AppliedServiceSetLoader::fetchForHost($host);
+        foreach ($appliedSets as $set) {
+            $title = sprintf($this->translate('%s (Applied Service set)'), $set->getObjectName());
+            $table = $this->loadTable('IcingaServiceSetService')
+                ->setServiceSet($set)
+                // ->setHost($host)
+                ->setAffectedHost($host)
+                ->setTitle($title)
+                ->setConnection($db);
+
+            $tables[$title] = $table;
         }
 
         $title = $this->translate('Applied services');
@@ -142,14 +156,19 @@ class HostController extends ObjectController
             ->setTitle($title)
             ->setConnection($db);
 
-        $tables[$title] = $table;
+        if (count($table)) {
+            $tables[$title] = $table;
+        }
 
         $this->view->tables = $tables;
     }
 
-    protected function addHostServiceSetTables(IcingaHost $host, & $tables)
+    protected function addHostServiceSetTables(IcingaHost $host, & $tables, IcingaHost $affectedHost = null)
     {
         $db = $this->db();
+        if ($affectedHost === null) {
+            $affectedHost = $host;
+        }
 
         $query = $db->getDbAdapter()->select()
             ->from(
@@ -166,12 +185,12 @@ class HostController extends ObjectController
             )->where('hs.host_id = ?', $host->id);
 
         $sets = IcingaServiceSet::loadAll($db, $query, 'object_name');
-
         foreach ($sets as $name => $set) {
             $title = sprintf($this->translate('%s (Service set)'), $name);
             $table = $this->loadTable('IcingaServiceSetService')
                 ->setServiceSet($set)
                 ->setHost($host)
+                ->setAffectedHost($affectedHost)
                 ->setTitle($title)
                 ->setConnection($db);
 
@@ -389,22 +408,31 @@ class HostController extends ObjectController
         $this->view->globalzone = $this->db()->getDefaultGlobalZoneName();
     }
 
+    protected function handleApiRequest()
+    {
+        // TODO: I hate doing this:
+        if ($this->getRequest()->getActionName() === 'ticket') {
+            $host = $this->object;
+
+            if ($host->getResolvedProperty('has_agent') !== 'y') {
+                throw new NotFoundError('The host "%s" is not an agent', $host->object_name);
+            }
+
+            return $this->sendJson(
+                Util::getIcingaTicket(
+                    $host->object_name,
+                    $this->api()->getTicketSalt()
+                )
+            );
+        }
+
+        return parent::handleApiRequest();
+    }
+
     public function ticketAction()
     {
-        if (! $this->getRequest()->isApiRequest() || ! $this->object) {
+        if (! $this->getRequest()->isApiRequest()) {
             throw new NotFoundError('Not found');
         }
-
-        $host = $this->object;
-        if ($host->getResolvedProperty('has_agent') !== 'y') {
-            throw new NotFoundError('The host "%s" is not an agent', $host->object_name);
-        }
-
-        return $this->sendJson(
-            Util::getIcingaTicket(
-                $host->object_name,
-                $this->api()->getTicketSalt()
-            )
-        );
     }
 }
