@@ -1,42 +1,36 @@
 <?php
 
-/**
- * This file ...
- *
- * @copyright  Icinga Team <team@icinga.org>
- * @license  GPLv2 http://www.gnu.org/licenses/gpl-2.0.html
- */
 namespace Icinga\Module\Director\Data\Db;
 
-use Icinga\Data\Db\DbConnection;
-use Icinga\Module\Director\Util;
 use Icinga\Exception\IcingaException as IE;
 use Icinga\Exception\NotFoundError;
+use Icinga\Module\Director\Exception\DuplicateKeyException;
+use Icinga\Module\Director\Util;
 use Exception;
+use Zend_Db_Adapter_Abstract;
 
 /**
  * Base class for ...
  */
 abstract class DbObject
 {
-    /**
-     * DbConnection
-     */
+    /** @var DbConnection $connection */
     protected $connection;
 
-    /**
-     * Zend_Db_Adapter_Abstract: DB Handle
-     */
+    /** @var string Table name. MUST be set when extending this class */
+    protected $table;
+
+    /** @var Zend_Db_Adapter_Abstract */
     protected $db;
 
-    /**
-     * Table name. MUST be set when extending this class
-     */
-    protected $table;
+    /** @var  Zend_Db_Adapter_Abstract */
+    protected $r;
 
     /**
      * Default columns. MUST be set when extending this class. Each table
      * column MUST be defined with a default value. Default value may be null.
+     *
+     * @var array
      */
     protected $defaultProperties;
 
@@ -234,6 +228,8 @@ abstract class DbObject
      *
      * @param string $property Property
      *
+     * @throws IE
+     *
      * @return mixed
      */
     public function get($property)
@@ -271,10 +267,12 @@ abstract class DbObject
     /**
      * Generic setter
      *
-     * @param string $property
+     * @param string $key
      * @param mixed  $value
      *
-     * @return array
+     * @throws IE
+     *
+     * @return self
      */
     public function set($key, $value)
     {
@@ -333,6 +331,8 @@ abstract class DbObject
     /**
      * Magic getter
      *
+     * @param mixed $key
+     *
      * @return mixed
      */
     public function __get($key)
@@ -356,6 +356,7 @@ abstract class DbObject
     /**
      * Magic isset check
      *
+     * @param  string $key
      * @return boolean
      */
     public function __isset($key)
@@ -366,6 +367,8 @@ abstract class DbObject
     /**
      * Magic unsetter
      *
+     * @param string $key
+     * @throws IE
      * @return void
      */
     public function __unset($key)
@@ -377,10 +380,10 @@ abstract class DbObject
     }
 
     /**
-     * Führt die Operation set() für jedes Element (key/value Paare) der über-
-     * gebenen Arrays aus
+     * Runs set() for every key/value pair of the given Array
      *
-     * @param  array  $data  Array mit den zu setzenden Daten
+     * @param  array $props  Array of properties
+     * @throws IE
      * @return self
      */
     public function setProperties($props)
@@ -567,6 +570,7 @@ abstract class DbObject
      * Lädt einen Datensatz aus der Datenbank und setzt die entsprechenden
      * Eigenschaften dieses Objekts
      *
+     * @throws NotFoundError
      * @return self
      */
     protected function loadFromDb()
@@ -575,7 +579,6 @@ abstract class DbObject
         $properties = $this->db->fetchRow($select);
 
         if (empty($properties)) {
-
             if (is_array($this->getKeyName())) {
                 throw new NotFoundError(
                     'Failed to load %s for %s',
@@ -650,7 +653,6 @@ abstract class DbObject
             $properties,
             $this->createWhere()
         );
-
     }
 
     /**
@@ -679,7 +681,9 @@ abstract class DbObject
     /**
      * Store object to database
      *
-     * @return boolean  Whether storing succeeded
+     * @param  DbConnection $db
+     * @return bool Whether storing succeeded
+     * @throws IE
      */
     public function store(DbConnection $db = null)
     {
@@ -714,7 +718,7 @@ abstract class DbObject
                 }
             } else {
                 if ($id && $this->existsInDb()) {
-                    throw new IE(
+                    throw new DuplicateKeyException(
                         'Trying to recreate %s (%s)',
                         $table,
                         $this->getLogId()
@@ -722,7 +726,6 @@ abstract class DbObject
                 }
 
                 if ($this->insertIntoDb()) {
-                    $id = $this->getId();
                     if ($this->autoincKeyName) {
                         if ($this->connection->isPgsql()) {
                             $this->properties[$this->autoincKeyName] = $this->db->lastInsertId(
@@ -732,9 +735,6 @@ abstract class DbObject
                         } else {
                             $this->properties[$this->autoincKeyName] = $this->db->lastInsertId();
                         }
-                        if (! $id) {
-                            $id = '[' . $this->properties[$this->autoincKeyName] . ']';
-                        }
                     }
                     // $this->log(sprintf('New %s "%s" has been stored', $table, $id));
                     $this->onInsert();
@@ -747,11 +747,11 @@ abstract class DbObject
                     );
                 }
             }
-
         } catch (Exception $e) {
             if ($e instanceof IE) {
                 throw $e;
             }
+
             throw new IE(
                 'Storing %s[%s] failed: %s {%s}',
                 $this->table,
@@ -760,6 +760,7 @@ abstract class DbObject
                 var_export($this->getProperties(), 1) // TODO: Remove properties
             );
         }
+
         $this->modifiedProperties = array();
         $this->hasBeenModified = false;
         $this->loadedProperties = $this->properties;
@@ -781,6 +782,11 @@ abstract class DbObject
         );
     }
 
+    /**
+     * @param string $key
+     * @return self
+     * @throws IE
+     */
     protected function setKey($key)
     {
         $keyname = $this->getKeyName();
@@ -920,10 +926,15 @@ abstract class DbObject
     {
     }
 
+    /**
+     * @param array $properties
+     * @param DbConnection|null $connection
+     *
+     * @return static
+     */
     public static function create($properties = array(), DbConnection $connection = null)
     {
-        $class = get_called_class();
-        $obj = new $class();
+        $obj = new static();
         if ($connection !== null) {
             $obj->setConnection($connection);
         }
@@ -931,11 +942,20 @@ abstract class DbObject
         return $obj;
     }
 
+    protected static function classWasPrefetched()
+    {
+        $class = get_called_class();
+        return array_key_exists($class, self::$prefetched);
+    }
+
     protected static function getPrefetched($key)
     {
         $class = get_called_class();
         if (static::hasPrefetched($key)) {
-            if (is_string($key) && array_key_exists($key, self::$prefetchedNames[$class])) {
+            if (is_string($key)
+                && array_key_exists($class, self::$prefetchedNames)
+                && array_key_exists($key, self::$prefetchedNames[$class])
+            ) {
                 return self::$prefetched[$class][
                     self::$prefetchedNames[$class][$key]
                 ];
@@ -949,8 +969,6 @@ abstract class DbObject
 
     protected static function hasPrefetched($key)
     {
-        // TODO: temporarily disabled as of collisions with services
-        //return false;
         $class = get_called_class();
         if (! array_key_exists($class, self::$prefetchStats)) {
             self::$prefetchStats[$class] = (object) array(
@@ -967,7 +985,10 @@ abstract class DbObject
         }
 
         if (array_key_exists($class, self::$prefetched)) {
-            if (is_string($key) && array_key_exists($key, self::$prefetchedNames[$class])) {
+            if (is_string($key)
+                && array_key_exists($class, self::$prefetchedNames)
+                && array_key_exists($key, self::$prefetchedNames[$class])
+            ) {
                 self::$prefetchStats[$class]->hitNames++;
                 return true;
             } elseif (array_key_exists($key, self::$prefetched[$class])) {
@@ -977,7 +998,6 @@ abstract class DbObject
                 self::$prefetchStats[$class]->miss++;
                 return false;
             }
-            return array_key_exists($key, self::$prefetched[$class]);
         } else {
             self::$prefetchStats[$class]->miss++;
             return false;
@@ -995,8 +1015,8 @@ abstract class DbObject
             return $prefetched;
         }
 
-        $class = get_called_class();
-        $obj = new $class();
+        /** @var DbObject $obj */
+        $obj = new static;
         $obj->setConnection($connection)
             ->set($obj->autoincKeyName, $id)
             ->loadFromDb();
@@ -1009,20 +1029,26 @@ abstract class DbObject
             return $prefetched;
         }
 
-        $class = get_called_class();
-        $obj = new $class();
+        /** @var DbObject $obj */
+        $obj = new static;
         $obj->setConnection($connection)->setKey($id)->loadFromDb();
         return $obj;
     }
 
+    /**
+     * @param DbConnection $connection
+     * @param \Zend_Db_Select $query
+     * @param string|null $keyColumn
+     *
+     * @return self[]
+     */
     public static function loadAll(DbConnection $connection, $query = null, $keyColumn = null)
     {
         $objects = array();
-        $class = get_called_class();
         $db = $connection->getDbAdapter();
 
         if ($query === null) {
-            $dummy = new $class();
+            $dummy = new static;
             $select = $db->select()->from($dummy->table);
         } else {
             $select = $query;
@@ -1031,7 +1057,8 @@ abstract class DbObject
         $rows = $db->fetchAll($select);
 
         foreach ($rows as $row) {
-            $obj = new $class();
+            /** @var DbObject $obj */
+            $obj = new static;
             $obj->setConnection($connection)->setDbProperties($row);
             if ($keyColumn === null) {
                 $objects[] = $obj;
@@ -1043,12 +1070,17 @@ abstract class DbObject
         return $objects;
     }
 
+    /**
+     * @param DbConnection $connection
+     * @param bool $force
+     *
+     * @return static[]
+     */
     public static function prefetchAll(DbConnection $connection, $force = false)
     {
-        $class = get_called_class();
-
-        $dummy = $class::create();
-        $autoInc = $dummy->getAutoIncKeyName();
+        $dummy = static::create();
+        $class = get_class($dummy);
+        $autoInc = $dummy->getAutoincKeyName();
         $keyName = $dummy->getKeyName();
 
         if ($force || ! array_key_exists($class, self::$prefetched)) {
@@ -1067,7 +1099,7 @@ abstract class DbObject
     {
         $class = get_called_class();
         if (! array_key_exists($class, self::$prefetched)) {
-            return false;
+            return;
         }
 
         unset(self::$prefetched[$class]);
@@ -1082,14 +1114,21 @@ abstract class DbObject
         self::$prefetchStats = array();
     }
 
+    /**
+     * @param $id
+     * @param DbConnection $connection
+     * @return bool
+     */
     public static function exists($id, DbConnection $connection)
     {
         if (static::getPrefetched($id)) {
             return true;
+        } elseif (static::classWasPrefetched()) {
+            return false;
         }
 
-        $class = get_called_class();
-        $obj = new $class();
+        /** @var DbObject $obj */
+        $obj = new static;
         $obj->setConnection($connection)->setKey($id);
         return $obj->existsInDb();
     }
