@@ -4,6 +4,8 @@ namespace Icinga\Module\Director\Import;
 
 use Exception;
 use Icinga\Data\Filter\Filter;
+use Icinga\Module\Director\Db;
+use Icinga\Module\Director\Db\Cache\PrefetchCache;
 use Icinga\Module\Director\Objects\IcingaObject;
 use Icinga\Module\Director\Objects\ImportSource;
 use Icinga\Module\Director\Objects\IcingaService;
@@ -19,6 +21,11 @@ class Sync
      */
     protected $rule;
 
+    /**
+     * @var Db
+     */
+    protected $db;
+
     /**
      * Related ImportSource objects
      *
@@ -60,14 +67,11 @@ class Sync
 
     protected $errors = array();
 
-    protected $hasCombinedKey;
-
-    protected $sourceKeyPattern;
-
-    protected $destinationKeyPattern;
-
     protected $syncProperties;
 
+    /**
+     * @var SyncRun
+     */
     protected $run;
 
     protected $runStartTime;
@@ -77,6 +81,8 @@ class Sync
     /**
      * Constructor. No direct initialization allowed right now. Please use one
      * of the available static factory methods
+     *
+     * @param SyncRule $rule
      */
     public function __construct(SyncRule $rule)
     {
@@ -106,7 +112,7 @@ class Sync
         foreach ($objects as $object) {
             if ($object->hasBeenModified()) {
                 $modified[] = $object;
-            } elseif ($object instanceof IcingaObject && $object->shouldBeRemoved()) {
+            } elseif ($object->shouldBeRemoved()) {
                 $modified[] = $object;
             }
         }
@@ -114,22 +120,6 @@ class Sync
         return $modified;
     }
 
-    /**
-     * Extract variable names in the form ${var_name} from a given string
-     *
-     * @param  string $string
-     *
-     * @return array  List of variable names (without ${})
-     */
-    protected function extractVariableNames($string)
-    {
-        if (preg_match_all('/\${([A-Za-z0-9\._-]+)}/', $string, $m, PREG_PATTERN_ORDER)) {
-            return $m[1];
-        } else {
-            return array();
-        }
-    }
-
     /**
      * Transform the given value to an array
      *
@@ -148,95 +138,6 @@ class Sync
         }
     }
 
-    /**
-     * Recursively extract a value from a nested structure
-     *
-     * For a $val looking like
-     *
-     * { 'vars' => { 'disk' => { 'sda' => { 'size' => '256G' } } } }
-     *
-     * and a key vars.disk.sda given as [ 'vars', 'disk', 'sda' ] this would
-     * return { size => '255GB' }
-     *
-     * @param  string $val  The value to extract data from
-     * @param  object $keys A list of nested keys pointing to desired data
-     *
-     * @return mixed
-     */
-    protected function getDeepValue($val, $keys)
-    {
-        $key = array_shift($keys);
-        if (! property_exists($val, $key)) {
-            return null;
-        }
-
-        if (empty($keys)) {
-            return $val->$key;
-        }
-
-        return $this->getDeepValue($val->$key, $keys);
-    }
-
-    /**
-     * Return a specific value from a given row object
-     *
-     * Supports also keys pointing to nested structures like vars.disk.sda
-     *
-     * @param  object $row    stdClass object providing property values
-     * @param  string $string Variable/property name
-     *
-     * @return mixed
-     */
-    public function getSpecificValue($row, $var)
-    {
-        if (strpos($var, '.') === false) {
-            if ($row instanceof IcingaObject) {
-                return $row->$var;
-            }
-            if (! property_exists($row, $var)) {
-                return null;
-            }
-
-            return $row->$var;
-        } else {
-            $parts = explode('.', $var);
-            $main = array_shift($parts);
-            if (! is_object($row->$main)) {
-                throw new IcingaException('Data is not nested, cannot access %s: %s', $var, var_export($row, 1));
-            }
-
-            return $this->getDeepValue($row->$main, $parts);
-        }
-    }
-
-    /**
-     * Fill variables in the given string pattern
-     *
-     * This replaces all occurances of ${var_name} with the corresponding
-     * property $row->var_name of the given row object. Missing variables are
-     * replaced by an empty string. This works also fine in case there are
-     * multiple variables to be found in your string.
-     *
-     * @param  string $string String with opional variables/placeholders
-     * @param  object $row    stdClass object providing property values
-     *
-     * @return string
-     */
-    protected function fillVariables($string, $row)
-    {
-        if (preg_match('/^\${([A-Za-z0-9\._-]+)}$/', $string, $m)) {
-            return $this->getSpecificValue($row, $m[1]);
-        }
-
-        // PHP 5.3 :(
-        $self = $this;
-        $func = function ($match) use ($self, $row) {
-            return $self->getSpecificValue($row, $match[1]);
-        };
-
-        return preg_replace_callback('/\${([A-Za-z0-9\._-]+)}/', $func, $string);
-    }
-
     /**
      * Raise PHP resource limits
      *
@@ -328,7 +229,7 @@ class Sync
                 $this->sourceColumns[$sourceId] = array();
             }
 
-            foreach ($this->extractVariableNames($p->source_expression) as $varname) {
+            foreach (SyncUtils::extractVariableNames($p->source_expression) as $varname) {
                 $this->sourceColumns[$sourceId][$varname] = $varname;
                 // -> ? $fieldMap[
             }
@@ -337,46 +238,6 @@ class Sync
         return $this;
     }
 
-    /**
-     * Whether we have a combined key (e.g. services on hosts)
-     *
-     * @return bool
-     */
-    protected function hasCombinedKey()
-    {
-        if ($this->hasCombinedKey === null) {
-
-            $this->hasCombinedKey = false;
-
-            if ($this->rule->object_type === 'service') {
-                $hasHost = false;
-                $hasObjectName = false;
-
-                foreach ($this->syncProperties as $key => $property) {
-                    if ($property->destination_field === 'host') {
-                        $hasHost = $property->source_expression;
-                    }
-                    if ($property->destination_field === 'object_name') {
-                        $hasObjectName = $property->source_expression;
-                    }
-                }
-
-                if ($hasHost !== false && $hasObjectName !== false) {
-                    $this->hasCombinedKey = true;
-                    $this->sourceKeyPattern = sprintf(
-                        '%s!%s',
-                        $hasHost,
-                        $hasObjectName
-                    );
-
-                    $this->destinationKeyPattern = '${host}!${object_name}';
-                }
-            }
-        }
-
-        return $this->hasCombinedKey;
-    }
-
     /**
      * Fetch latest imported data rows from all involved import sources
      *
@@ -386,33 +247,36 @@ class Sync
     {
         $this->imported = array();
 
+        $sourceKeyPattern = $this->rule->getSourceKeyPattern();
+        $combinedKey = $this->rule->hasCombinedKey();
+
         foreach ($this->sources as $source) {
+            /** @var ImportSource $source */
             $sourceId = $source->id;
 
             // Provide an alias column for our key. TODO: double-check this!
             $key = $source->key_column;
             $this->sourceColumns[$sourceId][$key] = $key;
-            $rows = $this->db->fetchLatestImportedRows(
-                $sourceId,
-                $this->sourceColumns[$sourceId]
+            $run = $source->fetchLastRun(true);
+            $rows = $run->fetchRows(
+                SyncUtils::getRootVariables($this->sourceColumns[$sourceId])
             );
 
             $this->imported[$sourceId] = array();
             foreach ($rows as $row) {
-                if ($this->hasCombinedKey()) {
-                    $key = $this->fillVariables($this->sourceKeyPattern, $row);
+                if ($combinedKey) {
+                    $key = SyncUtils::fillVariables($sourceKeyPattern, $row);
+
                     if (array_key_exists($key, $this->imported[$sourceId])) {
                         throw new IcingaException(
                             'Trying to import row "%s" (%s) twice: %s VS %s',
                             $key,
-                            $this->sourceKeyPattern,
+                            $sourceKeyPattern,
                             json_encode($this->imported[$sourceId][$key]),
                             json_encode($row)
                         );
                     }
-
                 } else {
-
                     if (! property_exists($row, $key)) {
                         throw new IcingaException(
                             'There is no key column "%s" in this row from "%s": %s',
@@ -421,19 +285,20 @@ class Sync
                             json_encode($row)
                         );
                     }
-
                 }
 
                 if (! $this->rule->matches($row)) {
                     continue;
                 }
 
-                if ($this->hasCombinedKey()) {
+                if ($combinedKey) {
                     $this->imported[$sourceId][$key] = $row;
                 } else {
                     $this->imported[$sourceId][$row->$key] = $row;
                 }
             }
+
+            unset($rows);
         }
 
         return $this;
@@ -457,7 +322,7 @@ class Sync
 
         $no = array();
         foreach ($this->objects as $k => $o) {
-            if ($o->list_id !== $listId) {
+            if ((int) $o->list_id !== (int) $listId) {
                 $no[] = $k;
             }
         }
@@ -470,30 +335,31 @@ class Sync
     protected function loadExistingObjects()
     {
         // TODO: Make object_type (template, object...) and object_name mandatory?
-        if ($this->hasCombinedKey()) {
-
+        if ($this->rule->hasCombinedKey()) {
             $this->objects = array();
+            $destinationKeyPattern = $this->rule->getDestinationKeyPattern();
 
             foreach (IcingaObject::loadAllByType(
                 $this->rule->object_type,
                 $this->db
             ) as $object) {
-
                 if ($object instanceof IcingaService) {
-                    if (! $object->host_id) {
+                    if (strstr($destinationKeyPattern, '${host}') && $object->host_id === null) {
+                        continue;
+                    } elseif (strstr($destinationKeyPattern, '${service_set}') && $object->service_set_id === null) {
                         continue;
                     }
                 }
 
-                $key = $this->fillVariables(
-                    $this->destinationKeyPattern,
+                $key = SyncUtils::fillVariables(
+                    $destinationKeyPattern,
                     $object
                 );
 
                 if (array_key_exists($key, $this->objects)) {
                     throw new IcingaException(
                         'Combined destination key "%s" is not unique, got "%s" twice',
-                        $this->destinationKeyPattern,
+                        $destinationKeyPattern,
                         $key
                     );
                 }
@@ -539,7 +405,7 @@ class Sync
 
                     $prop = $p->destination_field;
 
-                    $val = $this->fillVariables($p->source_expression, $row);
+                    $val = SyncUtils::fillVariables($p->source_expression, $row);
 
                     if (substr($prop, 0, 5) === 'vars.') {
                         $varName = substr($prop, 5);
@@ -582,7 +448,7 @@ class Sync
                 }
 
                 foreach ($newProps as $prop => $value) {
-                    // TODO: data type? 
+                    // TODO: data type?
                     $object->set($prop, $value);
                 }
 
@@ -613,6 +479,8 @@ class Sync
             return $this->objects;
         }
 
+        PrefetchCache::initialize($this->db);
+
         $this->raiseLimits()
              ->startMeasurements()
              ->fetchSyncProperties()
@@ -687,6 +555,8 @@ class Sync
         $dba = $db->getDbAdapter();
         $dba->beginTransaction();
 
+        $object = null;
+
         try {
             $formerActivityChecksum = Util::hex2binary(
                 $db->getLastActivityChecksum()
@@ -695,18 +565,7 @@ class Sync
             $modified = 0;
             $deleted = 0;
             foreach ($objects as $object) {
-                if ($object instanceof IcingaObject && $object->isTemplate()) {
-                    // TODO: allow to sync templates
-                    if ($object->hasBeenModified()) {
-                        throw new IcingaException(
-                            'Sync is not allowed to modify template "%s"',
-                            $object->$objectKey
-                        );
-                    }
-                    continue;
-                }
-
-                if ($object instanceof IcingaObject && $object->shouldBeRemoved()) {
+                if ($object->shouldBeRemoved()) {
                     $object->delete($db);
                     $deleted++;
                     continue;
@@ -744,10 +603,20 @@ class Sync
             $this->run->set('duration_ms', (int) round(
                 (microtime(true) - $this->runStartTime) * 1000
             ))->store();
-
         } catch (Exception $e) {
             $dba->rollBack();
-            throw $e;
+
+            if ($object !== null && $object instanceof IcingaObject) {
+                throw new IcingaException(
+                    'Exception while syncing %s %s: %s',
+                    get_class($object),
+                    $object->get('object_name'),
+                    $e->getMessage(),
+                    $e
+                );
+            } else {
+                throw $e;
+            }
         }
 
         return $this->run->id;
