@@ -22,6 +22,8 @@ class IcingaTimePeriod extends IcingaObject
 
     protected $supportsRanges = true;
 
+    protected $supportedInLegacy = true;
+
     protected $relations = array(
         'zone' => 'IcingaZone',
     );
@@ -37,7 +39,13 @@ class IcingaTimePeriod extends IcingaObject
     public function renderUpdate_method()
     {
         // @codingStandardsIgnoreEnd
-        return c::renderKeyValue('update', $this->update_method);
+        return '';
+    }
+
+    protected function renderObjectHeader()
+    {
+        return parent::renderObjectHeader()
+            . '    import "legacy-timeperiod"' . "\n";
     }
 
     public function isActive($now = null)
@@ -56,4 +64,9 @@ class IcingaTimePeriod extends IcingaObject
         //       different. Figure out whether and how we should support this
         return false;
     }
+
+    protected function prefersGlobalZone()
+    {
+        return true;
+    }
 }
